/*
 *    Copyright (C) 2002  Claudio Agosti - vecna@s0ftpj.org
 *                                         claudio@blackhats.it
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdarg.h>

#include "shared-func.h"

char *umpfkey;
int key_len;
static char keyinit =0x00;
/*
 * S is Secret table, P and Q two magic constant for table init
 * A and B swap vars, i j and k some counter.
 */
static unsigned char S[t], A, B;
static unsigned char P =0xb7, Q =0x9e;	
static int i, j, k;

/*
 * from RSA RC5 code, remplementation of algorithm for work as stream
 * chyper on single byte unit
 */
#define ROTL(x,y) (((x)<<(y%8)) | ((x)>>(8-(y%8))))
#define ROTR(x,y) (((x)>>(y%8)) | ((x)<<(8-(y%8))))

void RC5_like__ENCRYPT(byte *pt, byte *ct, off_t offset) 
	{ 
	A= *pt+S[0];

	for (i=1; i<=r; i++) 
		{
		A =ROTL(A, S[2*i]); 
		A =ROTL(A, S[(offset *i)%t]);
		}

	*ct =A;  
	} 

void RC5_like__DECRYPT(byte *ct, byte *pt, off_t offset) 
	{ 
	A= *ct;

	for (i=r; i>0; i--) 
		{
		A =ROTR(A, S[(offset *i)%t]);
		A =ROTR(A, S[2*i]);
		}

	*pt =A-S[0];
	} 

void RC5_like__SETUP()
	{
	byte L[key_len]; 

	keyinit++;

	memset((void *)&L, 0x00, key_len);

	/* Initialize L, then S, then mix key into S */
	for (i=key_len -1, L[key_len -1]=0; i!=-1; i--) 
		L[i] = (L[i]<<4)+umpfkey[i];

	for (S[0]=P, i=1; i<t; i++)
		S[i] = S[i-1]+Q;

	for (A=B=i=j=k=0; k<3*t; k++,i=(i+1)%t,j=(j+1)%key_len)
		{
		A = S[i] = ROTL(S[i]+(A+B),3);  
		B = L[j] = ROTL(L[j]+(A+B),(A+B)); 
		} 
	} 

/* shared function */
char *get_umpf_key(void)
	{
	struct sockaddr_un client;
	int fd;
	static char key_buff[KEYL];
	char *home;

	if((fd =socket(AF_UNIX, SOCK_STREAM, 0)) ==-1)
		fatal(USE_ERRNO, "unable to open unix socket for umpf srvr");

	if((home =getenv("HOME")) ==NULL)
		fatal(USE_ERRNO, "unable to read $HOME from enviroment");

	client.sun_family =AF_UNIX;
	snprintf(client.sun_path, UNIX_PATH_MAX, "%s/bin/umpf.unix", home);

	if(connect(fd, (struct sockaddr *)&client, sizeof(client)))
		fatal(USE_ERRNO, "unable to connect at %s", client.sun_path);

	if(read(fd, (void *)key_buff, KEYL) ==-1)
		fatal(USE_ERRNO, "unable to read %s", client.sun_path);

	close(fd);

	key_len =strlen(key_buff);
	
	return (char *)&key_buff;
	}

/* 
 * umpf_enc_buf and umpf_dec_buf is required for shared object
 * umpf-lib.c (-> umpf.so) for decrypt and encrypt buffer
 */
void umpf_enc_buf(char *buf, unsigned int nbytes, off_t offset)
	{
	unsigned int cnt;
	unsigned char swap;

	if(!keyinit)
		{
		if(umpfkey ==NULL)
			umpfkey =get_umpf_key();

		RC5_like__SETUP();
		}
	
	for(cnt =0; cnt < nbytes; cnt++)
		{
		RC5_like__ENCRYPT(&buf[cnt], &swap, offset +cnt);
		buf[cnt] =swap;
		}
	}

void umpf_dec_buf(char *buf, unsigned int nbytes, off_t offset)
	{
	unsigned int cnt;
	unsigned char swap;

	if(!keyinit)
		{
		if(umpfkey ==NULL)
			umpfkey =get_umpf_key();

		RC5_like__SETUP();
		}

	for(cnt =0; cnt < nbytes; cnt++)
		{
		RC5_like__DECRYPT(&buf[cnt], &swap, offset +cnt);
		buf[cnt] =swap;
		}
	}

void fatal(int use_errno, char *message, ...)
	{
	va_list va;
	va_start(va, message);

	if(use_errno)
		{
		vfprintf(stderr, message, va);
		fprintf(stderr," : %s\n", strerror(errno));
		}
	else
		{
		vfprintf(stderr, message, va);
		fprintf(stderr, "\n");
		}

	va_end(va);
	exit(errno);
	}

