/*
 *    Copyright (C) 2002  Claudio Agosti - vecna@s0ftpj.org
 *                                         claudio@blackhats.it
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdarg.h>
#include <time.h>

#include "shared-func.h"

#ifdef 	S_ISVTX
#ifndef S_ISTXT
#define S_ISTXT	S_ISVTX
#endif
#endif

static int usage(char *);
static void twist_umpf(struct stat, int);
static void new_umpf_empty(char *, struct stat);
static int mkumpftmp(char *, char *);

int main(int argc, char **argv)
	{
	int uid =getuid();
	int euid =geteuid();
	int gid =getgid();
	int egid =getegid();
	char *check =NULL, *key;

	if(argc == 1 || (argc == 2 && !strcmp(argv[1],"-h")))
		return usage(argv[0]);

	if((key =malloc(KEYL)) ==NULL)
		fatal(USE_ERRNO, "unable to alloc %d bytes", KEYL);

	if(!strcmp(argv[1], "-k"))
		{
		check =getpass("Insert umpf password: ");
		memcpy((void *)key, (void *)check, 128);

		check =getpass("reinsert please: ");

		if(memcmp((void *)key, (void *)check, 128))
			fatal(NOT_ERRNO, "password differ\n");

		argv++;
		}
	else
		key =get_umpf_key();

	fflush(stdout);
	
	while(*(argv +1) !=NULL)
		{
		char *file =*(++argv);
		struct stat info;
		int fd, i =4096;
		char buf[4096];
		char tmpfname[MAXPATHLEN];

		if(file[0] =='-')
			{
			if(file[1] !='-')
				printf(" %s read as option, ignored file\n",
					file);
			break;
			}
		
		if(stat(file, &info) ==-1)
			{
			if(errno ==ENOENT)
				new_umpf_empty(file, info);
			else
				perror(file);

			continue;
			}
		/* permission check */
		if(!S_ISREG(info.st_mode))
			{
			fprintf(stderr, "only regular file could be devish");
			fprintf(stderr, " %s rejected\n", file);

			continue;
			}
		if(info.st_uid !=uid && info.st_uid !=euid)
			{
			fprintf(stderr,"-- %s uid_t %d, your is %d/%d\n",
				file, info.st_uid, uid, euid);
			continue;
			}
		if(info.st_gid !=gid && info.st_gid !=egid)
			{
			fprintf(stderr,"-- %s gid_t %d, your is %d/%d\n",
				file, info.st_gid, gid, egid);
			continue;
			}

		if((fd =open(file, O_RDWR)) ==-1)
			{
			fprintf(stderr, "%s: ", file);
			perror("unable to open on read/write");
			}

		if(info.st_mode & S_ISTXT)
			{
			int tmpfd;
			
			tmpfd =mkumpftmp(file, (char *)&tmpfname);

			/* when i differ 4096 file is reach at EOF */
			while(i ==4096)
				{
				/*
				 * umpf_read is more faster and I use this 
				 * here because is required fast reading
				 * on conversion
				 */
				i =read(fd, (char *)&buf, 4096);
				if((write(tmpfd, (char *)&buf, i)) ==-1)
					fatal(USE_ERRNO, "write on tmp file");
				}

			/* close and unlink the ex-umpf file */
			close(fd);
			unlink(file);

			/* rename the new plain file */
			close(tmpfd);
			rename(tmpfname, file);
			}
		else
			{
			int tmpfd;
			
			tmpfd =mkumpftmp(file, (char *)&tmpfname);

			/*
			 * togg umpf flag for write on encrypted mode 
			 */
			twist_umpf(info, tmpfd);

			while(i ==4096)
				{
				i =read(fd, (char *)&buf, 4096);
				if((write(tmpfd, (char *)&buf, i)) ==-1)
					fatal(USE_ERRNO, "write on tmp file");
				}

			/* remove old plain file */
			unlink(file);
			close(fd);

			/* 
			 * set umpf flag (and retain old file attributes stored
			 * in struct stat info, rename and close file 
			 */
			close(tmpfd);
			rename(tmpfname, file);
			}
		}

	return(EXIT_SUCCESS);
	}
	
static int mkumpftmp(char *srcname, char *dstname)
	{
	int ret;

	snprintf(dstname, MAXPATHLEN, "%s%x", srcname, (int)time(NULL));

	if((ret =open(dstname, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR)) ==-1)
		fatal(USE_ERRNO, "unable to open tmp file %s", dstname);

	return ret;
	}


static void new_umpf_empty(char *file, struct stat info)
	{
	close(creat(file, info.st_mode =S_IRUSR|S_IWUSR));
	}

static void twist_umpf(struct stat info, int fd)
	{
	if(info.st_mode & S_ISTXT)
		fchmod(fd, S_IRUSR|S_IWUSR);
	else
		fchmod(fd, S_ISTXT|info.st_mode);

	fchown(fd, getuid(), getgid());
	}

static int usage(char *pname)
	{
	fprintf(stderr, "umpf file codec by vecna@s0ftpj.org - ");
	fprintf(stderr, "usage: <-k> %s files\n", pname);
	fprintf(stderr, "with -k option don't read ~/bin/umpf.unix\n");
	return(EXIT_FAILURE);
	}
